SystemOrganization addCategory: #'Sphero-Core'!

Object subclass: #Sphero
	instanceVariableNames: 'prot speed heading pitch roll yaw color stabilization'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sphero-Core'!

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 15:59'!
bitsigned16: a
	^ a > ((2 raisedTo: 16) / 2)
		ifTrue: [-1 * ((2 raisedTo: 16) - a)]
		ifFalse: a! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:08'!
color
	^ color! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:10'!
color: aColor
	color := aColor.
	prot 
		sendTo: 2
		command: 16r20
		data: {(aColor red * 255) asInteger.
				(aColor green * 255) asInteger.
				(aColor blue * 255) asInteger.
				0} asByteArray! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 15:53'!
connectOnPort: portName
	prot connectOnPort: portName.
	self isConnected ifTrue: [self initializeSampling]! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 15:53'!
disconnect
	prot disconnect! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:10'!
heading
	^ heading! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:11'!
heading: value
	heading := value \\ 360.
	heading < 0 ifTrue: [heading := 360 + heading].
	self sendRollCommand! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 15:53'!
idConnected
	^ prot isConnected! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 16:49'!
initialize
	super initialize.
	speed := heading := pitch := roll := yaw := 0.
	color := Color white.
	stabilization := true.
	prot := SpheroProtocol new
		when: #newData send: #updateData: to: self;
		yourself! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 15:56'!
initializeSampling
	prot
		sendTo: 2
		command: 16r11
		data: {0. "n1"
				16rF0. "n2"
				0. "m1"
				1. "m2"
				0. "mask1"
				7. "mask1 2"
				0. "mask1 3"
				0. "mask1 4"
				0. "packet limit"
				"0." "mask2"
				"0." "mask2 2"
				"1." "mask2 3"
				"16r80" "mask2 4"} asByteArray! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 16:24'!
isConnected
	^ prot isConnected ! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:08'!
pitch
	^ pitch! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:08'!
roll
	^ roll! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 15:58'!
sendRollCommand
	prot
		sendTo: 2 
		command: 16r30
		data: {speed.
				heading bitShift: -8.
				heading bitAnd: 255.
				1} asByteArray! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:11'!
speed
	^ speed! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:12'!
speed: value
	speed := value \\ 256.
	self sendRollCommand ! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:12'!
stabilization
	^ stabilization! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:15'!
stabilization: value
	stabilization := value.
	prot 
		sendTo: 2
		command: 16r2
		data: {value asBit} asByteArray! !

!Sphero methodsFor: 'as yet unclassified' stamp: 'Richo 7/31/2012 16:07'!
updateData: data
	(data at: 3) = 3
		ifTrue: [
			"Sensor(Data)"
			pitch := self bitsigned16: (((data at: 6) bitShift: 8) bitOr: (data at: 7)).
			roll := self bitsigned16: (((data at: 8) bitShift: 8) bitOr: (data at: 9)).
			yaw := self bitsigned16: (((data at: 10) bitShift: 8) bitOr: (data at: 11)).
			]! !

!Sphero methodsFor: 'accessing' stamp: 'Richo 7/31/2012 16:07'!
yaw
	^ yaw! !

Object subclass: #SpheroProtocol
	instanceVariableNames: 'port inQueue outQueue inTh outTh seq parsing state currentPacket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sphero-Core'!

!SpheroProtocol methodsFor: 'connection' stamp: 'Richo 7/31/2012 16:18'!
connectOnPort: aString
	self isConnected ifTrue: [^ self].
	port := SerialPort new baudRate: 57600.
	port := port
				openPort: (('COM*' match: aString)
						ifTrue: [aString asInteger]
						ifFalse: [aString]).
	self isConnected ifTrue: [self startProcesses]! !

!SpheroProtocol methodsFor: 'connection' stamp: 'Richo 7/31/2012 16:18'!
disconnect
	self isConnected ifFalse: [^ self].
	self stopProcesses.
	port close.
	port := nil.
	self initialize! !

!SpheroProtocol methodsFor: 'sending-receiving' stamp: 'Richo 7/31/2012 16:46'!
getCRC: packet
	"Assumes the argument is the whole packet with the space for the CRC,
	regardless of its value"
	| sum |
	sum := 0.
	3 to: packet size - 1 do: [:i | sum := sum + (packet at: i)].
	^ (sum \\ 256) bitXor: 2r11111111! !

!SpheroProtocol methodsFor: 'initialize-release' stamp: 'Richo 7/31/2012 16:31'!
initialize
	super initialize.
	inQueue := OrderedCollection new.
	outQueue := OrderedCollection new.
	state := #h1.
	parsing := false.
	seq := 0! !

!SpheroProtocol methodsFor: 'connection' stamp: 'Richo 7/31/2012 15:13'!
isConnected
	^ port notNil! !

!SpheroProtocol methodsFor: 'sending-receiving' stamp: 'Richo 7/31/2012 15:52'!
parsePacket
	parsing := true.
	[inQueue notEmpty]
		whileTrue: [| b |
			b := inQueue removeFirst.
			state caseOf: {
				[#h1] -> [b = 255
					ifTrue: [currentPacket := OrderedCollection with: b.
						state := #h2]].
				[#h2] -> [(b = 255 or: [b = 254])
					ifTrue: [currentPacket add: b.
						state := #MRSP]].
				[#MRSP] -> [currentPacket add: b.
						state := #SEQ].
				[#SEQ] -> [currentPacket add: b.
						state := #DLEN].
				[#DLEN] -> [currentPacket add: b.
						state := b = 1 ifTrue: [#CRC] ifFalse: [#DATA]].
				[#DATA] -> [currentPacket add: b.
						(currentPacket size - 5) = ((currentPacket at: 5) - 1)
							ifTrue: [state := #CRC]].
				[#CRC] -> [currentPacket add: b.
						(currentPacket last = (self getCRC: currentPacket))
							ifTrue: [self triggerEvent: #newData with: currentPacket asByteArray]
							ifFalse: [self error: 'CRC error'].
						state := #h1]
			}].
	parsing := false! !

!SpheroProtocol methodsFor: 'sending-receiving' stamp: 'Richo 7/31/2012 15:30'!
readInput
	self isConnected ifFalse: [^ self].
	inQueue addAllLast: port readByteArray.
	parsing ifFalse: [self parsePacket].
	10 milliSeconds asDelay wait! !

!SpheroProtocol methodsFor: 'sending-receiving' stamp: 'Richo 7/31/2012 15:25'!
sendTo: addr command: command data: data
	| buff |
	buff := {255.
			255.
			addr.
			command.
			seq.
			data size + 1},
			data,
			{0}.
	buff atLast: 1 put: (self getCRC: buff).
	outQueue addLast: buff asByteArray.
	seq := seq + 1.
	seq := seq \\ 256

	! !

!SpheroProtocol methodsFor: 'connection' stamp: 'Richo 7/31/2012 16:18'!
startProcesses
	inTh := [[self readInput] repeat] forkNamed: self asString , ' - In'.
	outTh := [[self writeOutput] repeat] forkNamed: self asString , ' - Out'.! !

!SpheroProtocol methodsFor: 'connection' stamp: 'Richo 7/31/2012 16:19'!
stopProcesses
	inTh ifNotNil: [:p | p terminate].
	outTh ifNotNil: [:p | p terminate].
	inTh := outTh := nil! !

!SpheroProtocol methodsFor: 'sending-receiving' stamp: 'Richo 7/31/2012 15:32'!
writeOutput
	self isConnected ifFalse: [^ self].
	outQueue notEmpty
		ifTrue: [port nextPutAll: outQueue removeFirst].
	10 milliSeconds asDelay wait! !
